<?php

namespace JimsCustomerBooking\Integrations;

use Nette\Mail\SmtpMailer;
use Nette\Mail\Message;

/**
 * This class is the base class for all notifications
 * Class ANotification
 *
 * @package JimsCustomerBooking\Integrations
 * @property string $service
 * @property string $email_from
 * @property string $email_to
 * @property string $encryption
 * @property SmtpMailer $service_provider
 * @property bool $test_mode
 * @property array $messages
 *
 * Example:
 * $email_notification = new EmailNotification('smtp_server', 'smtp_port', 'smtp_username', 'smtp_password', 'smtp_from', 'email_to', true);
 * $email_notification->setupNotificationMessages(['booking_id' => '123456', 'status' => 'success', 'error_code' => '0', 'website' => 'website']);
 * $email_notification->send(ANotification::NOTIFICATION_TYPE_NOTIFICATION);
 */
class EmailNotification extends ANotification
{

    protected string $service = self::NOTIFICATION_SERVICE_EMAIL;

    protected $email_from = '';

    protected $email_to = '';


    protected $encryption = SmtpMailer::EncryptionSSL;

    /**
     * EmailNotification constructor.
     *
     * @param $smtp_server
     * @param $smtp_port
     * @param $smtp_username
     * @param $smtp_password
     * @param $smtp_from
     * @param bool $test_mode
     */
    public function __construct($smtp_server, $smtp_port, $smtp_username, $smtp_password, $smtp_from, $email_to, bool $test_mode = false)
    {
        $this->test_mode = $test_mode;
        $this->email_from = $smtp_from;
        $this->email_to = $email_to;
        $this->setupService($smtp_server, $smtp_port, $smtp_username, $smtp_password);
    }

    /**
     * @param string $type
     * @param array $to
     * @param $from
     * @param $subject
     * @return bool
     */
    public function send(string $type = self::NOTIFICATION_TYPE_NOTIFICATION): bool
    {
        $message = $this->createMessage($type, $this->email_to, $this->email_from, $subject = "CB3 {$type} Notification");
        $sended = true;

        if ($this->service_provider instanceof SmtpMailer) {

            try {
                if($this->test_mode) {
                    error_log( $message->getBody() );
                }else{
                    $this->service_provider->send($message);
                }
            } catch (\Exception $e) {
                $sended = false;
                error_log($e->getMessage());
            }
        }

        return $sended;
    }

    /**
     * @param string $type
     * @param array $to
     * @param $from
     * @param $subject
     * @return Message
     */
    protected function createMessage(string $type, array $to, $from, $subject): Message
    {
        $message = $this->getMessage($type);

        $mail = new Message();
        $mail->setFrom($from);
        $mail->addTo($to);
        $mail->setSubject($subject);
        $mail->setHtmlBody($message);

        return $mail;
    }

    /**
     * @param $smtp_server
     * @param $smtp_port
     * @param $smtp_username
     * @param $smtp_password
     */
    private function setupService($smtp_server, $smtp_port, $smtp_username, $smtp_password)
    {
        $this->service_provider = new SmtpMailer(
            $smtp_server,
            $smtp_username,
            $smtp_password,
            $smtp_port,
            $this->encryption
        );
    }

    /**
     * @param array $vars, available keys: booking_id, status, error_code, website
     * @return void
     */
    public function setupNotificationMessages(array $vars = []): void
    {
        $notification = __( '{{icon}} *Booking ID: {{booking_id}} - {{status}} - Code 0* - on {{website}}');
        $info = __('{{icon}} *Booking ID: {{booking_id}} - {{status}} - Code 0* - on {{website}}');
        $error = __('{{icon}} *Booking ID: {{booking_id}} - {{status}} - Code {{error_code}}* - on {{website}}');


        $this->messages = [
            self::NOTIFICATION_TYPE_NOTIFICATION => 'Notification message',
            self::NOTIFICATION_TYPE_ERROR => 'Reminder message',
            self::NOTIFICATION_TYPE_INFO => 'Confirmation message',
        ];
    }
}